import visa


class InvalidChannelError(ValueError):
    pass


class K2600(object):
    """ Keithley 2600 instrument class. """
    def __init__(self, address=26):
        self.ctrl = visa.ResourceManager().open_resource( "GPIB::%s" % address )
        self.write = self.ctrl.write
        self.query = self.ctrl.query

        self.idn = self.query('*IDN?')

    def close(self):
        """ closes the VISA instance (I think) """
        self.ctrl.close()

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        """Close the Keithley"""
        self.close()

    def measure_current(self, channel='a'):
        """ queries instrument, returns value """
        channel = channel.lower()
        if channel in ('a', 'b'):
            return self.ctrl.query("print(smu{}.measure.i())".format(channel))
        else:
            raise InvalidChannelError()


    def measure_voltage(self, channel='a'):
        """ queries instrument, returns value """
        channel = channel.lower()
        if channel in ('a', 'b'):
            return self.ctrl.query("print(smu{}.measure.v())".format(channel))
        else:
            raise InvalidChannelError()


    def reset(self):
        """ resets instrument """
        self.ctrl.write( "reset()" )


    def reset_channel(self, channel='a'):
        """ resets VISA channel """
        channel = channel.lower()
        if channel in ('a', 'b'):
            self.ctrl.write( "smu{}.reset()".format(channel))
        else:
            raise ValueError("Invalid channel")


    def set_measure_current_range( self, meas_range=None, channel='a' ):
        """ if *meas_range* is not specified, measurement will be set to autorange """
        channel = channel.lower()
        if channel in ('a', 'b'):
            if meas_range is None:
                self.ctrl.write("smu{}.measure.autorangei = smua.AUTORANGE_ON".format(
                    channel
                ))
            else:                         
                self.ctrl.write("smu{}.measure.rangei = {}".format(
                    channel, meas_range
                ))
        else:
            raise ValueError("Invalid channel")
            
    def set_measure_voltage_range( self, meas_range=None, channel='A' ):
        """ if *meas_range* is not specified, measurement will be set to autorange """
        if channel=='A':
            if meas_range is None:
                self.ctrl.write(  "smua.source.autorangev = smua.AUTORANGE_ON" )
            else:                         
                self.ctrl.write(  "smua.source.rangev = %s" % meas_range )
        elif channel=='B':
            if meas_range is None:
                self.ctrl.write(  "smub.source.autorangev = smub.AUTORANGE_ON" )
            else:                         
                self.ctrl.write(  "smub.source.rangev = %s" % meas_range )
                print(meas_range)
        else:
            raise ValueError("Invalid channel sent to function set_measure_current_range")


    def set_current_compliance( self, compliance, channel='A' ):
        """ Set max output current level """
        if channel=='A':
            self.ctrl.write( "smua.source.limiti = %s" % compliance )
        elif channel=='B':
            self.ctrl.write( "smub.source.limiti = %s" % compliance )
        else:
            raise ValueError("Invalid channel sent to function set_current_compliance")

        
    def set_output_on( self, channel='A' ):
        """ turn on output """
        if channel=='A':
            self.ctrl.write( "smua.source.output = smua.OUTPUT_ON" )
        elif channel=='B':
            self.ctrl.write( "smub.source.output = smub.OUTPUT_ON" )
        else:
            raise ValueError("Invalid channel sent to function set_output_on")

        
    def set_output_off( self, channel='A' ):
        """ turn off output """
        if channel=='A':
            self.ctrl.write( "smua.source.output = smua.OUTPUT_OFF" )
        elif channel=='B':
            self.ctrl.write( "smub.source.output = smub.OUTPUT_OFF" )
        else:
            raise ValueError("Invalid channel sent to function set_output_off")

        
    def set_output_amps( self, amps, channel='A' ):
        """ set the output current in amps """
        if channel=='A':
            self.ctrl.write( "smua.source.leveli = %s" % amps )
        elif channel=='B':
            self.ctrl.write( "smub.source.leveli = %s" % amps )
        else:
            raise ValueError("Invalid channel sent to function set_output_amps")

        
    def set_output_volts( self, volts, channel='A' ):
        """ set the output voltage level in volts """
        if channel=='A':
            self.ctrl.write( "smua.source.levelv = %s" % volts )
        elif channel=='B':
            self.ctrl.write( "smub.source.levelv = %s" % volts )
        else:
            raise ValueError("Invalid channel sent to function set_output_volts")

        
    def set_source_type_current( self, src_range=None, channel='A' ):
        """
        set the smu channel to source current (in amps)
        if `src_range` is not specified, smu source will be set to autorange
        """
        if channel=='A':
            self.ctrl.write( "smua.source.func = smua.OUTPUT_DCAMPS" )
            if src_range is None:
                self.ctrl.write(  "smua.source.autorangei = smua.AUTORANGE_ON" )
            else:                         
                self.ctrl.write(  "smua.source.rangev = %s" % src_range )
        elif channel=='B':
            self.ctrl.write( "smub.source.func = smub.OUTPUT_DCAMPS" )
            if src_range is None:
                self.ctrl.write(  "smub.source.autorangei = smub.AUTORANGE_ON" )
            else:                         
                self.ctrl.write(  "smub.source.rangev = %s" % src_range )
        else:
            raise ValueError("Invalid channel sent to function set_source_type_current")


    def set_source_type_voltage( self, src_range=None, channel='A' ):
        """
        set the smu channel to source volts
        if `src_range` is not specified, smu source will be set to autorange
        """
        if channel=='A':
            self.ctrl.write( "smua.source.func = smua.OUTPUT_DCVOLTS" )
            if src_range is None:
                self.ctrl.write(  "smua.source.autorangev = smua.AUTORANGE_ON" )
            else:                         
                self.ctrl.write(  "smua.source.rangev = %s" % src_range )
        elif channel=='B':
            self.ctrl.write( "smub.source.func = smub.OUTPUT_DCVOLTS" )
            if src_range is None:
                self.ctrl.write(  "smub.source.autorangev = smub.AUTORANGE_ON" )
            else:                         
                self.ctrl.write(  "smub.source.rangev = %s" % src_range )
        else:
            raise ValueError("Invalid channel sent to function set_source_type_voltage")


    def write( self, command_string ):
        """ send the supplied string to the instrument """
        self.ctrl.write( command_string )

    def query(self, query):
        return self.ctl.query()



"""Minolta light-measuring devices
See http://www.konicaminolta.com/instruments

----------
"""
# Part of the PsychoPy library
# Copyright (C) 2015 Jonathan Peirce
# Distributed under the terms of the GNU General Public License (GPL).

import sys
import time

ABSOLUTE = '04'
PEAK = '08'
CONT = '09'


try:
    import serial
except ImportError:
    print("Could not load `serial` module, please install if possible.")
    sys.exit(1)


class LS100(object):
    """LS100 interface class
    """

    longName = "Minolta LS100/LS110"
    driverFor = ["ls110", "ls100"]
    codes = {
        'ER00\r\n': 'Unknown command',
        'ER01\r\n': 'Setting error',
        'ER11\r\n': 'Memory value error',
        'ER10\r\n': 'Measuring range over',
        'ER19\r\n': 'Display range over',
        'ER20\r\n': 'EEPROM error (the photometer needs repair)',
        'ER30\r\n': 'Photometer battery exhausted'
    }

    def __init__(self, port, maxAttempts=1):
        self.portString = port
        self.isOpen = 0
        self.lastQual = 0
        self.type = 'LS100'
        self.com = None
        self.OK = True
        self.maxAttempts = maxAttempts

        try:
            self.com = serial.Serial(self.portString)
        except:
            self._error(
                "Couldn't connect to port %s."
                "Is it being used by another program?".format(
                    self.portString
                )
            )

        # Setup the params for PR650 comms
        if self.OK:
            # Not sure why this helps but on win32 it does!!
            self.com.close()
            # This is a slightly odd characteristic of the Minolta LS100
            self.com.setByteSize(7)
            self.com.setBaudrate(4800)
            self.com.setParity(serial.PARITY_EVEN)  # none
            self.com.setStopbits(serial.STOPBITS_TWO)
            try:
                if not self.com.isOpen():
                    self.com.open()
            except:
                self._error(
                    "Opened serial port {},"
                    "but couldn't connect to LS100".format(
                        self.portString
                    )
                )
            else:
                self.isOpen = 1

        if self.OK:  # We have an open com port. try to send a command
            for repN in range(self.maxAttempts):
                time.sleep(0.2)
                for n in range(10):
                    # Set absolute mode
                    reply = self.sendMessage('MDS,04')
                    if reply[0:2] == 'OK':
                        self.OK = True
                        break
                    elif reply not in self.codes.keys():
                        self.OK = False
                        break  # wasn't valid
                    else:
                        self.OK = False  # false so far but keep trying

    @property
    def mode(self):
        return self.__mode

    @mode.setter
    def setMode(self, value=ABSOLUTE):
        self.__mode = value
        self._send('MDS,{}'.format(value))

    def measure(self):
        """Measure the current luminance and set .lastLum to this value"""
        reply = self._send('MES')
        if self._ok(reply):
            lum = float(reply.split()[-1])
            return lum
        else:
            return -1

    @property
    def lum(self):
        """Makes a measurement and returns the luminance value
        """
        return self.measure()

    def clear(self):
        """Clear the memory of the device from previous measurements
        """
        return self._ok(self._send('CLE'))

    def _ok(self, msg):
        return msg[:2] == 'OK'

    def _error(self, msg):
        self.OK = False

    def _send(self, msg, timeout=5.0):
        """Send a command to the photometer and wait an alloted
        timeout for a response.
        """
        if msg[-2:] != '\r\n':
            msg += '\r\n'  # append a newline if necess

        # flush the read buffer first
        self.com.read(self.com.inWaiting())
        for attemptN in range(self.maxAttempts):
            # Send the message
            time.sleep(0.1)
            self.com.write(msg)
            self.com.flush()
            time.sleep(0.1)
            # Get reply (within timeout limit)
            self.com.setTimeout(timeout)
            retVal = self.com.readline()
            if len(retVal) > 0:
                self.OK = self._ok(msg)
                return retVal



# pylint: disable=no-self-use
"""Tests for the Xtralien imports module

The imports module is used to provide a consistent user
interface to people using the Xtralien Python Distribution
and includes many common packages and functions.
"""
import unittest

from os import path
import numpy as np

import src.xtralien.imports as imports


class TestRunner(unittest.TestCase):
    """Test Runner class

    This class contains all of the tests for the
    xtralien imports modules
    """
    def test_load_csv_skip_headers(self):
        """Test loading a CSV by skipping headers
        """
        arr = imports.load_csv(
            path.join(
                path.dirname(path.realpath(__file__)),
                'data',
                'test_1.csv'
            ),
            skip_headers=True
        )

        self.assertEqual(
            arr.all(),
            (np.zeros((15, 9)) + 0.1).all()
        )

if __name__ == "__main__":
    unittest.main()



#/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Compat is a series of modules compatible with hardware brands other
than Xtralien.
The goal is to make creating any experiment as easy as using an
Xtralien Device.
"""



from setuptools import find_packages, setup

try:
    with open('README.md') as f:
        long_description = f.read()
except:
    long_description = ''

setup(
    name='xtralien',
    version='2.11.2',
    description='A library for controlling Ossila\'s Source Measure Unit',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Ossila Ltd.',
    author_email='info@ossila.com',
    url='https://github.com/xtralien/pyxtralien.git',
    packages=find_packages(where='src', exclude=['additional']),
    package_dir={'': 'src'},
    python_requires='>=3.5',
    extras_require={
        'numpy': ['numpy'],
        'Serial': ['pyserial'],
        'keithley': ['pyVISA'],
        'prompt': ['pyreadline']
    },
    license='MIT',
)
